@@ -8,7 +8,19 @@ import traceback
 from abc import ABC, abstractmethod
 from enum import Enum
 from functools import cached_property
-from typing import Any, Callable, Dict, Generic, List, Optional, Type, TypeVar, Union
+from typing import (
+    Any,
+    AsyncGenerator,
+    Callable,
+    Dict,
+    Generic,
+    List,
+    Literal,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+)
 from uuid import uuid4
 
 from opentelemetry import context as context_api
@@ -23,9 +35,11 @@ from opentelemetry.sdk.trace.export import (
 from opentelemetry.trace import Tracer
 from pydantic import BaseModel, Field
 
+from uipath._events._events import UiPathRuntimeEvent
 from uipath.agent.conversation import UiPathConversationEvent, UiPathConversationMessage
 from uipath.tracing import TracingManager
 
+from ..models.runtime_schema import BindingResource, Entrypoint
 from ._logging import LogsInterceptor
 
 logger = logging.getLogger(__name__)
@@ -79,6 +93,42 @@ class UiPathErrorCategory(str, Enum):
     USER = "User"  # Business logic or domain-level errors
 
 
+class UiPathErrorCode(str, Enum):
+    """Standard error codes for UiPath runtime errors."""
+
+    # Entrypoint related errors
+    ENTRYPOINT_MISSING = "ENTRYPOINT_MISSING"
+    ENTRYPOINT_NOT_FOUND = "ENTRYPOINT_NOT_FOUND"
+    ENTRYPOINT_FUNCTION_MISSING = "ENTRYPOINT_FUNCTION_MISSING"
+
+    # Module and execution errors
+    IMPORT_ERROR = "IMPORT_ERROR"
+    MODULE_EXECUTION_ERROR = "MODULE_EXECUTION_ERROR"
+    FUNCTION_EXECUTION_ERROR = "FUNCTION_EXECUTION_ERROR"
+    EXECUTION_ERROR = "EXECUTION_ERROR"
+
+    # Input validation errors
+    INVALID_INPUT_FILE_EXTENSION = "INVALID_INPUT_FILE_EXTENSION"
+    INPUT_INVALID_JSON = "INPUT_INVALID_JSON"
+
+    # Process and job related errors
+    INVOKED_PROCESS_FAILURE = "INVOKED_PROCESS_FAILURE"
+    API_CONNECTION_ERROR = "API_CONNECTION_ERROR"
+
+    # HITL (Human-In-The-Loop) related errors
+    HITL_FEEDBACK_FAILURE = "HITL_FEEDBACK_FAILURE"
+    UNKNOWN_HITL_MODEL = "UNKNOWN_HITL_MODEL"
+    HITL_ACTION_CREATION_FAILED = "HITL_ACTION_CREATION_FAILED"
+
+    # Trigger type errors
+    UNKNOWN_TRIGGER_TYPE = "UNKNOWN_TRIGGER_TYPE"
+
+    # Runtime shutdown errors
+    RUNTIME_SHUTDOWN_ERROR = "RUNTIME_SHUTDOWN_ERROR"
+
+    REFRESH_TOKEN_MISSING = "REFRESH_TOKEN_MISSING"
+
+
 class UiPathErrorContract(BaseModel):
     """Standard error contract used across the runtime."""
 
@@ -143,6 +193,19 @@ class UiPathRuntimeResult(BaseModel):
         return result
 
 
+class UiPathBreakpointResult(UiPathRuntimeResult):
+    """Result for execution suspended at a breakpoint."""
+
+    # Force status to always be SUSPENDED
+    status: UiPathRuntimeStatus = Field(
+        default=UiPathRuntimeStatus.SUSPENDED, frozen=True
+    )
+    breakpoint_node: str  # Which node the breakpoint is at
+    breakpoint_type: Literal["before", "after"]  # Before or after the node
+    current_state: dict[str, Any] | Any  # Current workflow state at breakpoint
+    next_nodes: List[str]  # Which node(s) will execute next
+
+
 class UiPathConversationHandler(ABC):
     """Base delegate for handling UiPath conversation events."""
 
@@ -314,10 +377,12 @@ class UiPathRuntimeContext(BaseModel):
     result: Optional[UiPathRuntimeResult] = None
     execution_output_file: Optional[str] = None
     input_file: Optional[str] = None
+    trace_file: Optional[str] = None
     is_eval_run: bool = False
     log_handler: Optional[logging.Handler] = None
     chat_handler: Optional[UiPathConversationHandler] = None
     is_conversational: Optional[bool] = None
+    breakpoints: Optional[List[str]] = None
 
     model_config = {"arbitrary_types_allowed": True, "extra": "allow"}
 
@@ -400,7 +465,7 @@ class UiPathRuntimeError(Exception):
 
     def __init__(
         self,
-        code: str,
+        code: UiPathErrorCode,
         title: str,
         detail: str,
         category: UiPathErrorCategory = UiPathErrorCategory.UNKNOWN,
@@ -419,8 +484,10 @@ class UiPathRuntimeError(Exception):
         if status is None:
             status = self._extract_http_status()
 
+        code_value = code.value
+
         self.error_info = UiPathErrorContract(
-            code=f"{prefix}.{code}",
+            code=f"{prefix}.{code_value}",
             title=title,
             detail=detail,
             category=category,
@@ -462,6 +529,12 @@ class UiPathRuntimeError(Exception):
         return self.error_info.model_dump()
 
 
+class UiPathRuntimeStreamNotSupportedError(NotImplementedError):
+    """Raised when a runtime does not support streaming."""
+
+    pass
+
+
 class UiPathBaseRuntime(ABC):
     """Base runtime class implementing the async context manager protocol.
 
@@ -484,6 +557,22 @@ class UiPathBaseRuntime(ABC):
         runtime = cls(context)
         return runtime
 
+    @property
+    def get_binding_resources(self) -> List[BindingResource]:
+        """Get binding resources for this runtime.
+
+        Returns: A list of binding resources.
+        """
+        raise NotImplementedError()
+
+    @property
+    def get_entrypoint(self) -> Entrypoint:
+        """Get entrypoint for this runtime.
+
+        Returns: A entrypoint for this runtime.
+        """
+        raise NotImplementedError()
+
     async def __aenter__(self):
         """Async enter method called when entering the 'async with' block.
 
@@ -497,7 +586,7 @@ class UiPathBaseRuntime(ABC):
             _, file_extension = os.path.splitext(self.context.input_file)
             if file_extension != ".json":
                 raise UiPathRuntimeError(
-                    code="INVALID_INPUT_FILE_EXTENSION",
+                    code=UiPathErrorCode.INVALID_INPUT_FILE_EXTENSION,
                     title="Invalid Input File Extension",
                     detail="The provided input file must be in JSON format.",
                 )
@@ -511,7 +600,7 @@ class UiPathBaseRuntime(ABC):
                 self.context.input_json = {}
         except json.JSONDecodeError as e:
             raise UiPathRuntimeError(
-                "INPUT_INVALID_JSON",
+                UiPathErrorCode.INPUT_INVALID_JSON,
                 "Invalid JSON input",
                 f"The input data is not valid JSON: {str(e)}",
                 UiPathErrorCategory.USER,
@@ -519,12 +608,14 @@ class UiPathBaseRuntime(ABC):
 
         await self.validate()
 
-        # Intercept all stdout/stderr/logs and write them to a file (runtime/evals), stdout (debug)
+        # Intercept all stdout/stderr/logs
+        # write to file (runtime) or stdout (debug)
         self.logs_interceptor = LogsInterceptor(
             min_level=self.context.logs_min_level,
             dir=self.context.runtime_dir,
             file=self.context.logs_file,
             job_id=self.context.job_id,
+            execution_id=self.context.execution_id,
             is_debug_run=self.is_debug_run(),
             log_handler=self.context.log_handler,
         )
@@ -546,6 +637,47 @@ class UiPathBaseRuntime(ABC):
         """
         pass
 
+    async def stream(
+        self,
+    ) -> AsyncGenerator[Union[UiPathRuntimeEvent, UiPathRuntimeResult], None]:
+        """Stream execution events in real-time.
+
+        This is an optional method that runtimes can implement to support streaming.
+        If not implemented, only the execute() method will be available.
+
+        Yields framework-agnostic BaseEvent instances during execution,
+        with the final event being UiPathRuntimeResult.
+
+        Yields:
+            UiPathRuntimeEvent subclasses: Framework-agnostic events (UiPathAgentMessageEvent,
+                                  UiPathAgentStateEvent, etc.)
+            Final yield: UiPathRuntimeResult (or its subclass UiPathBreakpointResult)
+
+        Raises:
+            UiPathRuntimeStreamNotSupportedError: If the runtime doesn't support streaming
+            RuntimeError: If execution fails
+
+        Example:
+            async for event in runtime.stream():
+                if isinstance(event, UiPathRuntimeResult):
+                    # Last event - execution complete
+                    print(f"Status: {event.status}")
+                    break
+                elif isinstance(event, UiPathAgentMessageEvent):
+                    # Handle message event
+                    print(f"Message: {event.payload}")
+                elif isinstance(event, UiPathAgentStateEvent):
+                    # Handle state update
+                    print(f"State updated by: {event.node_name}")
+        """
+        raise UiPathRuntimeStreamNotSupportedError(
+            f"{self.__class__.__name__} does not implement streaming. "
+            "Use execute() instead."
+        )
+        # This yield is unreachable but makes this a proper generator function
+        # Without it, the function wouldn't match the AsyncGenerator return type
+        yield
+
     @abstractmethod
     async def validate(self):
         """Validate runtime inputs."""
@@ -648,7 +780,7 @@ class UiPathBaseRuntime(ABC):
             raise
         finally:
             # Restore original logging
-            if self.logs_interceptor:
+            if hasattr(self, "logs_interceptor"):
                 self.logs_interceptor.teardown()
 
             await self.cleanup()
@@ -697,6 +829,7 @@ class UiPathRuntimeFactory(Generic[T, C]):
         self.context_generator = context_generator
         self.tracer_provider: TracerProvider = TracerProvider()
         self.tracer_span_processors: List[SpanProcessor] = []
+        self.logs_exporter: Optional[Any] = None
         trace.set_tracer_provider(self.tracer_provider)
 
     def add_span_exporter(
@@ -730,9 +863,9 @@ class UiPathRuntimeFactory(Generic[T, C]):
             return self.context_generator(**kwargs)
         return self.context_class(**kwargs)
 
-    def new_runtime(self) -> T:
+    def new_runtime(self, **kwargs) -> T:
         """Create a new runtime instance."""
-        context = self.new_context()
+        context = self.new_context(**kwargs)
         if self.runtime_generator:
             return self.runtime_generator(context)
         return self.runtime_class.from_context(context)
@@ -752,6 +885,28 @@ class UiPathRuntimeFactory(Generic[T, C]):
                 for span_processor in self.tracer_span_processors:
                     span_processor.force_flush()
 
+    async def stream(
+        self, context: C
+    ) -> AsyncGenerator[Union[UiPathRuntimeEvent, UiPathRuntimeResult], None]:
+        """Stream runtime execution with context.
+
+        Args:
+            context: The runtime context
+
+        Yields:
+            UiPathRuntimeEvent instances during execution and final UiPathRuntimeResult
+
+        Raises:
+            UiPathRuntimeStreamNotSupportedError: If the runtime doesn't support streaming
+        """
+        async with self.from_context(context) as runtime:
+            try:
+                async for event in runtime.stream():
+                    yield event
+            finally:
+                for span_processor in self.tracer_span_processors:
+                    span_processor.force_flush()
+
     async def execute_in_root_span(
         self,
         context: C,
@@ -777,6 +932,44 @@ class UiPathRuntimeFactory(Generic[T, C]):
                 for span_processor in self.tracer_span_processors:
                     span_processor.force_flush()
 
+    async def stream_in_root_span(
+        self,
+        context: C,
+        root_span: str = "root",
+        attributes: Optional[dict[str, str]] = None,
+    ) -> AsyncGenerator[Union[UiPathRuntimeEvent, UiPathRuntimeResult], None]:
+        """Stream runtime execution with context in a root span.
+
+        Args:
+            context: The runtime context
+            root_span: Name of the root span
+            attributes: Optional attributes to add to the span
+
+        Yields:
+            UiPathRuntimeEvent instances during execution and final UiPathRuntimeResult
+
+        Raises:
+            UiPathRuntimeStreamNotSupportedError: If the runtime doesn't support streaming
+        """
+        async with self.from_context(context) as runtime:
+            try:
+                tracer: Tracer = trace.get_tracer("uipath-runtime")
+                span_attributes = {}
+                if context.execution_id:
+                    span_attributes["execution.id"] = context.execution_id
+                if attributes:
+                    span_attributes.update(attributes)
+
+                with tracer.start_as_current_span(
+                    root_span,
+                    attributes=span_attributes,
+                ):
+                    async for event in runtime.stream():
+                        yield event
+            finally:
+                for span_processor in self.tracer_span_processors:
+                    span_processor.force_flush()
+
 
 class UiPathExecutionTraceProcessorMixin:
     def on_start(
