@@ -1,13 +1,15 @@
 # type: ignore
 import asyncio
 import os
-from typing import Any
+from typing import Any, AsyncIterator, Optional
 from urllib.parse import urlparse
 
 import click
 
+from uipath.models.exceptions import EnrichedException
+
 from ..telemetry import track
-from ._push.sw_file_handler import SwFileHandler
+from ._push.sw_file_handler import FileOperationUpdate, SwFileHandler
 from ._utils._console import ConsoleLogger
 from ._utils._constants import (
     UIPATH_PROJECT_ID,
@@ -32,17 +34,23 @@ def get_org_scoped_url(base_url: str) -> str:
 
 async def upload_source_files_to_project(
     project_id: str,
-    config_data: dict[Any, str],
+    settings: Optional[dict[str, Any]],
     directory: str,
     include_uv_lock: bool = True,
-) -> None:
-    """Upload source files to UiPath project.
+) -> AsyncIterator[FileOperationUpdate]:
+    """Upload source files to UiPath project, yielding progress updates.
 
     This function handles the pushing of local files to the remote project:
     - Updates existing files that have changed
     - Uploads new files that don't exist remotely
     - Deletes remote files that no longer exist locally
     - Optionally includes the UV lock file
+
+    Yields:
+        FileOperationUpdate: Progress updates for each file operation
+
+    Raises:
+        ProjectPushError: If the push operation fails
     """
     sw_file_handler = SwFileHandler(
         project_id=project_id,
@@ -50,7 +58,8 @@ async def upload_source_files_to_project(
         include_uv_lock=include_uv_lock,
     )
 
-    await sw_file_handler.upload_source_files(config_data)
+    async for update in sw_file_handler.upload_source_files(settings):
+        yield update
 
 
 @click.command()
@@ -88,21 +97,29 @@ def push(root: str, nolock: bool) -> None:
     config = get_project_config(root)
     validate_config(config)
 
-    if not os.getenv(UIPATH_PROJECT_ID, False):
+    project_id = os.getenv(UIPATH_PROJECT_ID)
+    if not project_id:
         console.error("UIPATH_PROJECT_ID environment variable not found.")
 
-    with console.spinner("Pushing coded UiPath project to Studio Web..."):
+    async def push_with_updates():
+        """Wrapper to handle async iteration and display updates."""
+        async for update in upload_source_files_to_project(
+            project_id,
+            config.get("settings", {}),
+            root,
+            include_uv_lock=not nolock,
+        ):
+            console.info(update.message)
+
+    with console.spinner("Pushing UiPath project to Studio Web..."):
         try:
             if not nolock:
                 handle_uv_operations(root)
 
-            asyncio.run(
-                upload_source_files_to_project(
-                    os.getenv(UIPATH_PROJECT_ID),
-                    config,
-                    root,
-                    include_uv_lock=not nolock,
-                )
-            )
+            asyncio.run(push_with_updates())
+
         except Exception as e:
-            console.error(f"Failed to push UiPath project: {e}")
+            console.error(
+                f"Failed to push UiPath project: {e}",
+                include_traceback=not isinstance(e, EnrichedException),
+            )
