@@ -6,19 +6,15 @@ from typing import Optional
 
 import click
 
+from uipath._cli._runtime._runtime_factory import generate_runtime_factory
 from uipath._cli._utils._debug import setup_debugging
-from uipath.tracing import LlmOpsHttpExporter
+from uipath.tracing import JsonLinesFileExporter, LlmOpsHttpExporter
 
 from .._utils.constants import (
     ENV_JOB_ID,
 )
 from ..telemetry import track
-from ._runtime._contracts import (
-    UiPathRuntimeContext,
-    UiPathRuntimeError,
-    UiPathRuntimeFactory,
-)
-from ._runtime._runtime import UiPathScriptRuntime
+from ._runtime._contracts import UiPathRuntimeError
 from ._utils._console import ConsoleLogger
 from .middlewares import Middlewares
 
@@ -48,6 +44,12 @@ console = ConsoleLogger()
     type=click.Path(exists=False),
     help="File path where the output will be written",
 )
+@click.option(
+    "--trace-file",
+    required=False,
+    type=click.Path(exists=False),
+    help="File path where the trace spans will be written (JSON Lines format)",
+)
 @click.option(
     "--debug",
     is_flag=True,
@@ -67,10 +69,20 @@ def run(
     file: Optional[str],
     input_file: Optional[str],
     output_file: Optional[str],
+    trace_file: Optional[str],
     debug: bool,
     debug_port: int,
 ) -> None:
     """Execute the project."""
+    context_args = {
+        "entrypoint": entrypoint,
+        "input": input,
+        "resume": resume,
+        "input_file": file or input_file,
+        "execution_output_file": output_file,
+        "trace_file": trace_file,
+        "debug": debug,
+    }
     input_file = file or input_file
     # Setup debugging if requested
     if not setup_debugging(debug, debug_port):
@@ -83,6 +95,7 @@ def run(
         resume,
         input_file=input_file,
         execution_output_file=output_file,
+        trace_file=trace_file,
         debug=debug,
         debug_port=debug_port,
     )
@@ -100,22 +113,22 @@ def run(
     Usage: `uipath run <entrypoint_path> <input_arguments> [-f <input_json_file_path>]`""")
 
         try:
-            runtime_factory = UiPathRuntimeFactory(
-                UiPathScriptRuntime, UiPathRuntimeContext
-            )
 
-            context = UiPathRuntimeContext.with_defaults(
-                entrypoint=entrypoint,
-                input=input,
-                input_file=input_file,
-                resume=resume,
-                execution_output_file=output_file,
-                debug=debug,
-            )
-            if context.job_id:
-                runtime_factory.add_span_exporter(LlmOpsHttpExporter())
+            async def execute() -> None:
+                runtime_factory = generate_runtime_factory()
+                context = runtime_factory.new_context(**context_args)
+                if context.job_id:
+                    runtime_factory.add_span_exporter(LlmOpsHttpExporter())
 
-            asyncio.run(runtime_factory.execute(context))
+                if trace_file:
+                    runtime_factory.add_span_exporter(JsonLinesFileExporter(trace_file))
+
+                result = await runtime_factory.execute(context)
+
+                if not context.job_id:
+                    console.info(result.output)
+
+            asyncio.run(execute())
 
         except UiPathRuntimeError as e:
             console.error(f"{e.error_info.title} - {e.error_info.detail}")
