@@ -1,4 +1,3 @@
-from os import environ as env
 from typing import Optional
 
 from pydantic import ValidationError
@@ -13,6 +12,7 @@ from ._services import (
     BucketsService,
     ConnectionsService,
     ContextGroundingService,
+    DocumentsService,
     EntitiesService,
     FolderService,
     JobsService,
@@ -21,12 +21,8 @@ from ._services import (
     UiPathLlmChatService,
     UiPathOpenAIService,
 )
-from ._utils import setup_logging
-from ._utils.constants import (
-    ENV_BASE_URL,
-    ENV_UIPATH_ACCESS_TOKEN,
-    ENV_UNATTENDED_USER_ACCESS_TOKEN,
-)
+from ._utils._auth import resolve_config
+from ._utils._logs import setup_logging
 from .models.errors import BaseUrlMissingError, SecretMissingError
 
 
@@ -36,19 +32,18 @@ class UiPath:
         *,
         base_url: Optional[str] = None,
         secret: Optional[str] = None,
+        client_id: Optional[str] = None,
+        client_secret: Optional[str] = None,
+        scope: Optional[str] = None,
         debug: bool = False,
     ) -> None:
-        base_url_value = base_url or env.get(ENV_BASE_URL)
-        secret_value = (
-            secret
-            or env.get(ENV_UNATTENDED_USER_ACCESS_TOKEN)
-            or env.get(ENV_UIPATH_ACCESS_TOKEN)
-        )
-
         try:
+            base_url, secret = resolve_config(
+                base_url, secret, client_id, client_secret, scope
+            )
             self._config = Config(
-                base_url=base_url_value,  # type: ignore
-                secret=secret_value,  # type: ignore
+                base_url=base_url,
+                secret=secret,
             )
         except ValidationError as e:
             for error in e.errors():
@@ -59,8 +54,8 @@ class UiPath:
         self._folders_service: Optional[FolderService] = None
         self._buckets_service: Optional[BucketsService] = None
         self._attachments_service: Optional[AttachmentsService] = None
-
-        setup_logging(debug)
+        self._connections_service: Optional[ConnectionsService] = None
+        setup_logging(should_debug=debug)
         self._execution_context = ExecutionContext()
 
     @property
@@ -97,7 +92,15 @@ class UiPath:
 
     @property
     def connections(self) -> ConnectionsService:
-        return ConnectionsService(self._config, self._execution_context)
+        if not self._connections_service:
+            if not self._folders_service:
+                self._folders_service = FolderService(
+                    self._config, self._execution_context
+                )
+            self._connections_service = ConnectionsService(
+                self._config, self._execution_context, self._folders_service
+            )
+        return self._connections_service
 
     @property
     def context_grounding(self) -> ContextGroundingService:
@@ -114,6 +117,10 @@ class UiPath:
             self._buckets_service,
         )
 
+    @property
+    def documents(self) -> DocumentsService:
+        return DocumentsService(self._config, self._execution_context)
+
     @property
     def queues(self) -> QueuesService:
         return QueuesService(self._config, self._execution_context)
